/*=====================================================================*/
/*                                                                     */
/*       FILE NAME:        euler.c                                     */
/*                                                                     */
/*       PROGRAM NAME:     euler    (prog. 2.1)                        */
/*                                                                     */
/*                                                                     */
/*   Version 1.00                                                      */
/*                                                                     */
/*   (c) Satoru Ozawa and Dieter W. Heermann                           */
/*   Hitachi, Heidelberg July 1994                                     */
/*                                                                     */
/*=====================================================================*/

/***********************************************************************/
/* one-dimensional harmonic oscillator                                 */
/* Euler-method                                                        */
/***********************************************************************/

#include <stdio.h>
#include "sgl.h"               /* include files for graphics       */

#define  dt      0.05          /* time interval of an unit MD step */
#define  maxmds  1000          /* maximum number of MD steps       */
#define  m       1.            /* mass of particle                 */
#define  k       1.            /* force constant                   */
#define  r       0.05          /* radius of disc                   */

void  main(int argc, char *argv[])
{
  float  x, v, xnew, vnew, e;
  int    mds, i;

  sgl_winsize(512,512);        /* display-size in pixels */
  sgl_init();                  /* initialize graphics    */

  x = 1.;                      /* initial conditions     */
  v = 0.;

  for (mds = 1; mds <= maxmds ; mds++){  /* integration of equations */
                                         /* of motion                */
    vnew = v - k/m*x*dt;
    xnew = x + v*dt;
    v = vnew;
    x = xnew;
    
    e = m/2.*v*v + k/2.*x*x;                         /* total energy */
    printf("mds=%d, x=%f, e=%f \n", mds, x, e);

    sgl_disc(x*0.1+0.5, 0.5, r);                   /* display motion */ 
    sgl_display();  
  }
  i = getchar();                /* hold picture until hit of any key */
}

