/*=====================================================================*/
/*                                                                     */
/*       FILE NAME:        filein.c                                      */
/*                                                                     */
/*       PROGRAM NAME:     filein  (Prog. 2.7)                           */
/*                                                                     */
/*                                                                     */
/*   Version 1.00                                                      */
/*                                                                     */
/*   (c) Satoru Ozawa and Dieter W. Heermann                           */
/*   Hitachi, Heidelberg July 1994                                     */
/*                                                                     */
/*=====================================================================*/

/***********************************************************************/
/* analysis of chaos                                                   */
/***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "sgl.h"                     /* include files for graphics  */

#define  n             64            /* number of particles         */
#define  maxmds        20000         /* maximum number of MD steps  */
#define  msamp         20            /* sampling interval           */
#define  file_name_1   "xv1.data"    /* name of input files         */
#define  file_name_2   "xv2.data"

void  main()
{
  int   i, j, jmax;
  float x1[n+1], v1[n+1];            /* position and velocity of    */
  float x2[n+1], v2[n+1];            /* particles                   */
  float m[n+1], k[n+1];              /* masses and force constants  */
  float dd;                          /* distance between two states */
  double log(double z);

  FILE    *fh_in_1;                  /* pointers to input files     */
  FILE    *fh_in_2;

  sgl_winsize(800,800);              /* display-size in pixels      */
  sgl_init();                        /* initialize graphics         */
  sgl_color(SGL_BLACK);              /* select color for drawings   */
  sgl_color_background(SGL_WHITE);   /* select color for background */

  jmax = maxmds/msamp;

  if ((fh_in_1 = fopen(file_name_1, "r")) == NULL){  /* open input file */
    fprintf(stderr, "error opening file: %s \n", file_name_1);
    fclose(fh_in_1);
    return;
  }

  if ((fh_in_2 = fopen(file_name_2, "r")) == NULL){  /* open input file */
    fprintf(stderr, "error opening file: %s \n", file_name_2);
    fclose(fh_in_2);
    return;
  }

  for(i=1; i<=n; i++){                               /* read file  */
    fscanf(fh_in_1, "%f, %f, ", &m[i], &k[i]);
    fscanf(fh_in_2, "%f, %f, ", &m[i], &k[i]);
  }

  for(j=1; j<=jmax; j++){  
    dd = 0.;
    for(i=1; i<=n; i++){                             /* read file  */
      fscanf(fh_in_1, "%f, %f, ", &x1[i], &v1[i]);
      fscanf(fh_in_2, "%f, %f, ", &x2[i], &v2[i]);

      dd += (x1[i]-x2[i])*(x1[i]-x2[i])
            +m[i]*m[i]*(v1[i]-v2[i])*(v1[i]-v2[i]);
    }
                                                    /* draw graph */
    sgl_disc(0.0008*j+0.1, 0.1*(float)log(dd)+0.6, 0.004);

    printf("j=%d, dd=%f, log(dd)=%f \n", j, dd, (float)log(dd));
  }

  sgl_line(0.1, 0.1*(float)log(0.01)+0.5,           /* draw axes  */
           0.9, 0.1*(float)log(0.01)+0.5);
  sgl_line(0.1, 0.1*(float)log(0.1)+0.5,
           0.9, 0.1*(float)log(0.1)+0.5);
  sgl_line(0.1, 0.1*(float)log(1.)+0.5,
           0.9, 0.1*(float)log(1.)+0.5);
  sgl_line(0.1, 0.1*(float)log(10.)+0.5,
           0.9, 0.1*(float)log(10.)+0.5);
  sgl_line(0.1, 0.1*(float)log(100.)+0.5,
           0.9, 0.1*(float)log(100.)+0.5);

  sgl_line(0.1, 0.1*(float)log(0.01)+0.5,
           0.1, 0.1*(float)log(100.)+0.5);
  sgl_line(0.3, 0.1*(float)log(0.01)+0.5,
           0.3, 0.1*(float)log(100.)+0.5);
  sgl_line(0.5, 0.1*(float)log(0.01)+0.5,
           0.5, 0.1*(float)log(100.)+0.5);
  sgl_line(0.7, 0.1*(float)log(0.01)+0.5,
           0.7, 0.1*(float)log(100.)+0.5);
  sgl_line(0.9, 0.1*(float)log(0.01)+0.5,
           0.9, 0.1*(float)log(100.)+0.5);

  sgl_display();                      /* display graph */

  fclose(fh_in_1);                    /* close files   */
  fclose(fh_in_2);

  i = getchar();                      /* hold display  */
}
