/*=====================================================================*/
/*                                                                     */
/*       FILE NAME:        runge.c                                     */
/*                                                                     */
/*       PROGRAM NAME:     runge    (Prog. 2.2)                       */
/*                                                                     */
/*                                                                     */
/*   Version 1.00                                                      */
/*                                                                     */
/*   (c) Satoru Ozawa and Dieter W. Heermann                           */
/*   Hitachi, Heidelberg July 1994                                     */
/*                                                                     */
/*=====================================================================*/

/***********************************************************************/
/* one-dimensional harmonic oscillator                                 */
/* Runge-Kutta method                                                  */
/***********************************************************************/

#include <stdio.h>
#include "sgl.h"                /* include files for graphics       */

#define  dt      0.05	        /* time interval of an unit MD step */	
#define  maxmds  1000           /* maximum number of MD steps       */
#define  m       1.             /* mass of particle                 */
#define  k       1.             /* force constant                   */
#define  r       0.05           /* radius of disc                   */

void	main(int argc, char *argv[])
{
  float  x, v, xnew, vnew, e;
  float  s, ss, sss, ssss;
  float  a, aa, aaa, aaaa;
  int    mds, i;

  sgl_winsize(512,512);    /* display-size in pixels */
  sgl_init();              /* initialize graphics    */

  x = 1.;                  /* initial condition      */
  v = 0.;

  for (mds = 1; mds <= maxmds ; mds++){  /* integration of equation  */
                                         /* of motion                */
    a = -k/m * x;
    s = v;
    aa = -k/m * (x+s*dt/2.);
    ss = v + a*dt/2.;
    aaa = -k/m * (x+ss*dt/2.);
    sss = v + aa*dt/2.;
    aaaa = -k/m * (x+sss*dt);
    ssss = v + aaa*dt;
    vnew = v + (a+2.*aa+2.*aaa+aaaa)*dt/6.;
    xnew = x + (s+2.*ss+2.*sss+ssss)*dt/6.;
    v = vnew;
    x = xnew;
  
    e = m/2.*v*v + k/2.*x*x;                        /* total energy */
    printf("mds=%d, x=%f, e=%f \n", mds, x, e);

    sgl_disc(x*0.1+0.5, 0.5, r);                  /* display motion */
    sgl_display();
  }
  i = getchar();               /* hold picture until hit of any key */
}
