/************************************************************************/
/*									*/
/*	LIBSGL.C				(C) 9/94 Ch. Muenkel	*/
/*	Main part of SGL library -- basic subroutines			*/
/*									*/
/************************************************************************/
/*									*/
/*	Simple Graphics Library						*/
/*	An easy to use interface to the X11 window system		*/
/*									*/
/************************************************************************/
/*									*/
/*	Author:	Christian Muenkel					*/
/*		Present address:					*/
/*		muenkel@carbo.tphys.uni-heidelberg.de			*/
/*		Christian Muenkel					*/
/*		Institut fuer Theoretische Physik			*/
/*		Philosophenweg 19					*/
/*		69120 Heidelberg - Germany				*/
/*									*/
/************************************************************************/
/* version      date            comment                                 */
/*                                                                      */
/* V1.0		 1/94		first implementation 			*/
/*		 9/94		final tests				*/
/*                                                                      */
/************************************************************************/

#define		__SGL_MAIN
#include	"sglglob.h"
#undef		__SGL_MAIN

#ifdef	__STDC__
static	void	sgl_create_colmap(void);
static	void	sgl_priv_init(int num_win);
#else
static void	sgl_create_colmap();
static	void	sgl_priv_init();
#endif



/************************************************************************/
/************************************************************************/
/*									*/
/* INITIALIZATIONS ...							*/
/*									*/
/************************************************************************/
/************************************************************************/


/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_init(void)						*/
/*		initialize X11 graphics					*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void	sgl_init(void)
#else
void	sgl_init()
#endif
{
  sgl_priv_init(1);
}

#ifdef	__STDC__
void	sgl_initm(int	num_win)
#else
void	sgl_initm(num_win)
	int	num_win;
#endif
{
  sgl_priv_init(num_win);
}

#ifdef	__STDC__
static void	sgl_priv_init(int	num_win)
#else
static void	sgl_priv_init(num_win)
	int	num_win;
#endif
{
  int		i,mask;
  XVisualInfo	visinf;
  XSetWindowAttributes attr;
  unsigned long gc_valmask;
  XGCValues	gcval;
  char		str[1024];

  if (flag_initialized == True)
     return;

  GD_num = num_win;
  if (GD_num <= 0)			/* nothing to initialize	*/
   { FatalError("SGL_INIT: argument must be positive.\n");
   }
  else if (GD_num > MAX_X11GD_STRUCT)	/* more than we can do		*/
   { FatalError("SGL_INIT: Too many windows.\n");
   }
	
  win_x_min = 0.0;
  win_x_max = 1.0;
  win_y_min = 0.0;
  win_y_max = 1.0;
  win_idx = 1.0 / (win_x_max - win_x_min);
  win_idy = 1.0 / (win_y_max - win_y_min);

  /***** allocate memory ************************************************/
  GD_act = 0;
  for (i = 0; i < GD_num; i++)
   { X11GD[i]=(struct X11GlobalData *)malloc(sizeof(struct X11GlobalData));
     if (X11GD[i] == NULL)
     FatalError("SGL_INIT: Can't allocate X11 global data structure.\n");
     X11GD[i]->win_sx = win_sx;
     X11GD[i]->win_sy = win_sy;
     X11GD[i]->win_x_min = win_x_min;
     X11GD[i]->win_y_min = win_y_min;
     X11GD[i]->win_x_max = win_x_max;
     X11GD[i]->win_y_max = win_y_max;
     X11GD[i]->win_idx = win_idx;
     X11GD[i]->win_idy = win_idy;
   }

  /***** allocate color table ***********************************************/
  color_pixel = (unsigned long *) calloc(COLOR_MAX,sizeof(unsigned long));
  if (color_pixel == NULL)
  FatalError("SGL_INIT: Can't allocate color data structure.\n");

  /***** Open X window display ******************************************/
  X11GD[0]->disp_name = NULL;
  X11GD[0]->disp = XOpenDisplay( X11GD[0]->disp_name );
  if ( ! X11GD[0]->disp )
   { sprintf(str,"Can't open display %s .\n",
		XDisplayName(X11GD[0]->disp_name));
     FatalError(str);
   }
  X11GD[0]->scrnn = DefaultScreen( X11GD[0]->disp);
  X11GD[0]->scrn  = ScreenOfDisplay( X11GD[0]->disp, X11GD[0]->scrnn);
  i = 1;
  while (i < GD_num)
   { if (X11GD[0]->disp_name == NULL)
       X11GD[i]->disp_name = X11GD[0]->disp_name;
     else
       strcpy(X11GD[i]->disp_name,X11GD[0]->disp_name);
     X11GD[i]->disp  = X11GD[0]->disp;
     X11GD[i]->scrnn = X11GD[0]->scrnn;
     X11GD[i]->scrn  = X11GD[0]->scrn ;
     i++;
   }

  /***** Detect color model used by display ******************************/
  /***** first, check for best: TrueColor display ************************/
  if ( XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			24, TrueColor, &visinf))
   { X11GD[0]->vistype = VISTYPE_TrueColor;
     X11GD[0]->vis = visinf.visual;
     X11GD[0]->depth = visinf.depth;
     sgl_create_colmap();			/* create colormap	*/
     mask = 0;
     attr.colormap = X11GD[0]->colmap;
     mask |= CWColormap;
     attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
     mask |= CWBorderPixel;
     attr.backing_store = Always;
     mask |= CWBackingStore;
     X11GD[0]->win = XCreateWindow(X11GD[0]->disp,
		RootWindow(X11GD[0]->disp,X11GD[0]->scrnn),
		0,0, win_sx,win_sy, 1, visinf.depth, InputOutput,
		X11GD[0]->vis, mask, &attr);
     XStoreName(X11GD[0]->disp, X11GD[0]->win, "SGL TrueColor");
     X11GD[0]->pm = XCreatePixmap(X11GD[0]->disp,X11GD[0]->win,
		win_sx,win_sy,visinf.depth);
     if (!X11GD[0]->pm) 
       FatalError("SGL_INIT: Couldn't create TrueColor X pixmap.");
     gc_valmask = 0;
     X11GD[0]->gc = XCreateGC( X11GD[0]->disp, X11GD[0]->win,
			gc_valmask, &gcval);
   }
  /***** second, check for PseudoColor display ************************/
  else if (XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
				8,PseudoColor,&visinf))
   { X11GD[0]->vistype = VISTYPE_PseudoColor;
     X11GD[0]->vis = visinf.visual;
     X11GD[0]->depth = visinf.depth;
     sgl_create_colmap();			/* create colormap	*/
     mask = 0;
     attr.colormap = X11GD[0]->colmap;
     mask |= CWColormap;
     attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
     mask |= CWBorderPixel;
     attr.backing_store = Always;
     mask |= CWBackingStore;
     X11GD[0]->win = XCreateWindow(X11GD[0]->disp, 
		RootWindow(X11GD[0]->disp, X11GD[0]->scrnn),
		0,0, win_sx,win_sy, 1, visinf.depth, InputOutput,
		X11GD[0]->vis, mask, &attr);
     XStoreName(X11GD[0]->disp, X11GD[0]->win, "SGL PseudoColor");
     X11GD[0]->pm = XCreatePixmap(X11GD[0]->disp,X11GD[0]->win,
		win_sx,win_sy,visinf.depth);
     if (!X11GD[0]->pm) 
       FatalError("SGL_INIT: Couldn't create PseudoColor X pixmap.");
     gc_valmask = 0;
     X11GD[0]->gc = XCreateGC(X11GD[0]->disp, X11GD[0]->win,
				gc_valmask, &gcval);
   }
  /***** third, check for StaticGray, GrayScale display ****************/
  else if (XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			1,StaticGray,&visinf)  ||
           XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			1,GrayScale,&visinf))
   { if (XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
				1,StaticGray,&visinf))
        X11GD[0]->vistype = VISTYPE_StaticGray;
     else if (XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
				1,GrayScale,&visinf))
        X11GD[0]->vistype = VISTYPE_GrayScale;
     X11GD[0]->depth = visinf.depth;
     mask = 0;
     attr.colormap = X11GD[0]->colmap;
     mask |= CWColormap;
     attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
     mask |= CWBorderPixel;
     attr.backing_store = Always;
     mask |= CWBackingStore;
     X11GD[0]->win = XCreateWindow(X11GD[0]->disp,
		RootWindow(X11GD[0]->disp, X11GD[0]->scrnn),
		0,0, win_sx,win_sy, 1, visinf.depth, InputOutput,
		X11GD[0]->vis, mask, &attr);
     XStoreName(X11GD[0]->disp, X11GD[0]->win, "SGL Gray/BW");
     X11GD[0]->pm = XCreatePixmap(X11GD[0]->disp,X11GD[0]->win,
		win_sx,win_sy,visinf.depth);
     if (!X11GD[0]->pm) 
       FatalError("SGL_INIT: Couldn't create StaticGray/GrayScale X pixmap.");
     gc_valmask = 0;
     X11GD[0]->gc = XCreateGC(X11GD[0]->disp, X11GD[0]->win,
				gc_valmask, &gcval);
     color_pixel[0]= BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
     color_pixel[1]= WhitePixel(X11GD[0]->disp,X11GD[0]->scrnn);
     color_stack_top = 2;
   }
  else
   { FatalError("SGL_INIT: Sorry, this type of Visual can't be used.\n");
   }

  i = 1;
  while (i < GD_num)			/* initialize second ... window	*/
   { X11GD[i]->vistype = X11GD[0]->vistype;
     X11GD[i]->vis     = X11GD[0]->vis;
     X11GD[i]->depth   = X11GD[0]->depth;
     X11GD[i]->colmap  = X11GD[0]->colmap;
     mask = 0;
     attr.colormap = X11GD[0]->colmap;
     mask |= CWColormap;
     attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
     mask |= CWBorderPixel;
     attr.backing_store = Always;
     mask |= CWBackingStore;
     X11GD[i]->win = XCreateWindow(X11GD[0]->disp, 
		RootWindow(X11GD[0]->disp, X11GD[0]->scrnn),
		0,0, win_sx,win_sy, 1, X11GD[0]->depth, InputOutput,
		X11GD[0]->vis, mask, &attr);
     sprintf(str,"Window %d",i+1);
     XStoreName(X11GD[i]->disp, X11GD[i]->win, str);
     X11GD[i]->pm = XCreatePixmap(X11GD[i]->disp,X11GD[i]->win,
		win_sx,win_sy,X11GD[i]->depth);
     if (!X11GD[i]->pm) 
       FatalError("SGL_INIT: Couldn't create PseudoColor X pixmap.");
     gc_valmask = 0;
     X11GD[i]->gc = XCreateGC( X11GD[i]->disp, X11GD[i]->win,
			gc_valmask, &gcval);
     i++;
   }

  for (i = 0; i < GD_num; i++)
   { XMapRaised(X11GD[i]->disp, X11GD[i]->win);
     XMoveWindow(X11GD[i]->disp, X11GD[i]->win,win_px,win_py);
     XSetWindowColormap( X11GD[i]->disp, X11GD[i]->win,
	X11GD[i]->colmap);
   }

  if (debugging == True) 
   { switch(X11GD[0]->vistype)
      { case VISTYPE_GrayScale:
	fprintf(stderr,"SGL: display type is GrayScale.\n");
	break;
	case VISTYPE_StaticGray:
	fprintf(stderr,"SGL: display type is StaticGray.\n");
	break;
	case VISTYPE_PseudoColor:
	fprintf(stderr,"SGL: display type is PseudoColor.\n");
	break;
	case VISTYPE_TrueColor:
	fprintf(stderr,"SGL: display type is TrueColor.\n");
	break;
	default:
	fprintf(stderr,"SGL: display type is unknown.\n");
	break;
      }
   }

  for (i = 0; i < GD_num; i++)
   { XSync(X11GD[i]->disp, X11GD[i]->win);
     XSetWindowColormap(X11GD[i]->disp, X11GD[i]->win, X11GD[i]->colmap);

     XSetBackground( X11GD[i]->disp, X11GD[i]->gc, color_pixel[0]);
     XSetForeground( X11GD[i]->disp, X11GD[i]->gc, color_pixel[0]);
     XFillRectangle( X11GD[i]->disp, X11GD[i]->pm, X11GD[i]->gc,
		0,0,win_sx,win_sy);
     XDrawLine( X11GD[i]->disp, X11GD[i]->pm, X11GD[i]->gc,
		0,0,win_sx,win_sy);
     XCopyArea( X11GD[i]->disp, X11GD[i]->pm, X11GD[i]->win,
		X11GD[i]->gc, 0,0,win_sx,win_sy,0,0);
     XSetForeground( X11GD[i]->disp, X11GD[i]->gc, color_pixel[1]);
     XFlush(X11GD[i]->disp);
     XSync(X11GD[i]->disp, X11GD[i]->win);
   }

  color_background = 0;
  color_foreground = 1;
  flag_initialized = True;

}


/************************************************************************/
/************************************************************************/
/*									*/
/* DRAWING FUNCTIONS							*/
/*									*/
/************************************************************************/
/************************************************************************/

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_line(x1,y1,x2,y2)					*/
/*		draw a line from (x1,y1) to (x2,y2) in viewport coord.	*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_line(float x1, float y1, float x2, float y2)
#else
void	sgl_line(x1,y1,x2,y2)
	float	x1,y1,x2,y2;
#endif
{
  int	ix1,iy1,ix2,iy2;

  ix1 = ((x1 - win_x_min) * win_idx) * win_sx;
  ix2 = ((x2 - win_x_min) * win_idx) * win_sx;
  iy1 = win_sy - ((y1 - win_y_min) * win_idy) * win_sy;
  iy2 = win_sy - ((y2 - win_y_min) * win_idy) * win_sy;

  if (flag_initialized == True)
    XDrawLine(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	ix1,iy1,ix2,iy2);
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_circle(x,y,r)					*/
/*		draw a circle at (x,y) with radius r in viewport coord	*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_circle(float x, float y, float r)
#else
void	sgl_circle(x,y,r)
	float	x,y,r;
#endif
{
  int	ix,iy,ir;

  ix = (((x-r) - win_x_min) * win_idx) * win_sx;
  iy = win_sy - (((y+r) - win_y_min) * win_idy) * win_sy;
  ir = (r * win_idx) * win_sx * 2;

  if (flag_initialized == True)
    XDrawArc(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	ix,iy,ir,ir,0,64*360);
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_disc(x,y,r)						*/
/*		draw a filled circle at (x,y) with radius r in viewport	*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_disc(float x, float y, float r)
#else
void	sgl_disc(x,y,r)
	float	x,y,r;
#endif
{
  int	ix,iy,ir;

  ix = (((x-r) - win_x_min) * win_idx) * win_sx;
  iy = win_sy - (((y+r) - win_y_min) * win_idy) * win_sy;
  ir = (r * win_idx) * win_sx * 2;

  if (flag_initialized == True)
    XFillArc(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	ix,iy,ir,ir,0,64*360);
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_rectangle(x,y,dx,dy)				*/
/*		draw a rectangle; (x,y) is the lower-right corner	*/
/*		(dx,dy) are the width/ height of the rectangle		*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_rectangle(float x, float y, float dx, float dy)
#else
void	sgl_rectangle(x,y,dx,dy)
	float	x,y,dx,dy;
#endif
{
  int	ix,iy,idx,idy;

  ix  = ((x - win_x_min) * win_idx) * win_sx;
  idx = (dx * win_idx) * win_sx;
  iy  = win_sy - (((y+dy) - win_y_min) * win_idy) * win_sy;
  idy = (dy * win_idy) * win_sy;

  if (flag_initialized == True)
    XDrawRectangle(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	ix,iy,idx,idy);
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_rectarea(x,y,dx,dy)					*/
/*		draw a filled rectangle; (x,y) is the lower-right	*/
/*		corner, (dx,dy) are the width/ height of the rectangle	*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_rectarea(float x, float y, float dx, float dy)
#else
void	sgl_rectarea(x,y,dx,dy)
	float	x,y,dx,dy;
#endif
{
  int	ix,iy,idx,idy;

  ix  = ((x - win_x_min) * win_idx) * win_sx;
  idx = (dx * win_idx) * win_sx;
  iy  = win_sy - (((y+dy) - win_y_min) * win_idy) * win_sy;
  idy = (dy * win_idy) * win_sy;

  if (flag_initialized == True)
    XFillRectangle(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	ix,iy,idx,idy);
}

  

/************************************************************************/
/************************************************************************/
/*									*/
/* Modifiers 								*/
/*									*/
/************************************************************************/
/************************************************************************/

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_winsize(sx,sy)					*/
/*		redefine window size in pixels				*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_winsize(int new_sx, int new_sy)
#else
void	sgl_winsize(new_sx,new_sy)
	int	new_sx,new_sy;
#endif
{
 if (flag_initialized != True)			/* not initialized	*/
  { win_sx = new_sx;
    win_sy = new_sy;
  }
}


/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_viewport(xmin,ymin,dx,dy)				*/
/*		redefine viewport coordinate system			*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_viewport(float xmin, float ymin, float dx, float dy)
#else
void	sgl_viewport(xmin,ymin,dx,dy)
	float	xmin,ymin,dx,dy;
#endif
{ 
  int	flag,new_sx,new_sy;

  if (fabs((win_idy/win_idx) - (dx/dy)) > 1E-3)
    flag = 1;				/* window aspect ratio changed	*/ 
  else
    flag = 0;
  win_x_min = xmin;
  win_y_min = ymin;
  win_x_max = xmin + dx;
  win_y_max = ymin + dy;
  win_idx = 1.0 / dx;
  win_idy = 1.0 / dy;
  new_sx = win_sx;
  new_sy = win_sx * (dy/dx);
  if (flag != 0)
    sgl_winsize(new_sx,new_sy);
}


/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_display(void)					*/
/*		display background drawings (double buffer)		*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__
void		sgl_display(void)
#else
void		sgl_display()
#endif
{
  XCopyArea(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
	X11GD[GD_act]->win,X11GD[GD_act]->gc,
	0,0,win_sx,win_sy,0,0);
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc,
	color_pixel[color_background]);
  XFillRectangle(X11GD[GD_act]->disp, X11GD[GD_act]->pm,
	X11GD[GD_act]->gc, 0,0,win_sx,win_sy);
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc,
	color_pixel[color_foreground]);
  XFlush(X11GD[GD_act]->disp);
  XSync(X11GD[GD_act]->disp, X11GD[GD_act]->win);
}

/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_pos_win(int x, int y)                               */
/*              position a created window on screen                     */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_pos_win(int x, int y)
#else
void    sgl_pos_win(x,y)
	int	x,y;
#endif
{
  XMoveWindow(X11GD[GD_act]->disp, X11GD[GD_act]->win,x,y);
}

/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_select_win(int win)                                 */
/*              select a created X11 window for drawing                 */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_select_win(int win)
#else
void    sgl_select_win(win)
	int	win;
#endif
{
 if (win < 0 || win >= GD_num)
  { fprintf(stderr,"SGL_SELECT_WIN: Selected window (%d) out of range\n",win);
    return;
  }
 else
  { GD_act = win;
  }
}


/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_map_win(int win)                                    */
/*              map a created window onto screen                        */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_map_win(int win)
#else
void    sgl_map_win(win)
	int	win;
#endif
{
 if (win < 0 || win >= GD_num)
  { fprintf(stderr,"SGL_MAP_WIN: Selected window (%d) out of range\n",win);
    return;
  }
 else
  { XMapWindow(X11GD[win]->disp, X11GD[win]->win);
  }
}

/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_unmap_win(int win)                                  */
/*              unmap a created window                                  */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_unmap_win(int win)
#else
void    sgl_unmap_win(win)
	int	win;
#endif
{
 if (win < 0 || win >= GD_num)
  { fprintf(stderr,"SGL_UNMAP_WIN: Selected window (%d) out of range\n",win);
    return;
  }
 else
  { XUnmapWindow(X11GD[win]->disp, X11GD[win]->win);
  }
}




/************************************************************************/
/************************************************************************/
/*									*/
/* COLOR FUNCTIONS							*/
/*									*/
/************************************************************************/
/************************************************************************/

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_create_colmap(void)					*/
/*		create (pseudo) color colormap				*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
static void	sgl_create_colmap(void)
#else
static void	sgl_create_colmap()
#endif
{
  XColor	exactColor,actualColor;
  int		ret;

  X11GD[GD_act]->colmap = 0;
  /***** Use default colormap if possible *****/
  if (X11GD[GD_act]->vis == DefaultVisualOfScreen( X11GD[GD_act]->scrn ) )
   { X11GD[GD_act]->colmap = DefaultColormapOfScreen( X11GD[GD_act]->scrn );
   }
  else
   { X11GD[GD_act]->colmap = XCreateColormap(X11GD[GD_act]->disp, 
		RootWindow(X11GD[GD_act]->disp,X11GD[GD_act]->scrnn),
		X11GD[GD_act]->vis, AllocNone);
   }
  if (X11GD[GD_act]->colmap == 0) 
    FatalError("SGL_CREATE_COLMAP: Can't create color map for visual.\n");

  ret = XAllocNamedColor(X11GD[GD_act]->disp, X11GD[GD_act]->colmap, "black",
	&actualColor,&exactColor);
  if (ret == 0)
    fprintf(stderr,"sgl_create_colmap: Can't allocate 'black' color.\n");
  color_pixel[0] = actualColor.pixel;
  ret = XAllocNamedColor(X11GD[GD_act]->disp, X11GD[GD_act]->colmap, "white",
	&actualColor,&exactColor);
  if (ret == 0)
    fprintf(stderr,"sgl_create_colmap: Can't allocate 'white' color.\n");
  color_pixel[1] = actualColor.pixel;
  color_stack_top = 2;
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_color(int)						*/
/*		choose actual drawing color				*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_color(int colindex)
#else
void	sgl_color(colindex)
	int	colindex;
#endif
{
  switch(X11GD[GD_act]->vistype)
   { case VISTYPE_GrayScale:
     case VISTYPE_StaticGray:
     case VISTYPE_PseudoColor:
     case VISTYPE_TrueColor:
	if (colindex < 0)
	  fprintf(stderr,"SGL_COLOR: There are no negativ (%d) colors.\n",
		colindex);
	else if (colindex >= color_stack_top)
	  fprintf(stderr,"SGL_COLOR: color %d not defined.\n",colindex);
	else
	 { XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc,
		color_pixel[colindex]);
	   color_foreground = colindex;
	 }
	break;
   }
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_color_background(int)				*/
/*		choose actual background color				*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_color_background(int colindex)
#else
void	sgl_color_background(colindex)
	int	colindex;
#endif
{
  switch(X11GD[GD_act]->vistype)
   { case VISTYPE_GrayScale:
     case VISTYPE_StaticGray:
     case VISTYPE_PseudoColor:
     case VISTYPE_TrueColor:
	if (colindex < 0)
	  fprintf(stderr,"SGL_COLOR_BACKGROUND: No negativ (%d) colors.\n",
		colindex);
	else if (colindex >= color_stack_top)
	  fprintf(stderr,"SGL_COLOR_BACKGROUND: color %d not defined.\n",
		colindex);
	else
	 { XSetBackground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
		color_pixel[colindex]);
	   color_background = colindex;
	 }
	break;
   }
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
	color_pixel[color_background]);
  XFillRectangle(X11GD[GD_act]->disp, X11GD[GD_act]->pm, X11GD[GD_act]->gc,
	0,0,win_sx,win_sy);
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
	color_pixel[color_foreground]);
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_colordef(int, char *)				*/
/*		define a colormap entry					*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
int	sgl_colordef(char *colname)
#else
int	sgl_colordef(colname)
	char	*colname;
#endif
{
  XColor	exactColor,actualColor;
  int		ret,return_value;

  return_value = -1;
  switch(X11GD[GD_act]->vistype)
   { case VISTYPE_GrayScale:
	return_value = 1;
	break;
     case VISTYPE_StaticGray:
	return_value = 1;
	break;
     case VISTYPE_PseudoColor:
     case VISTYPE_TrueColor:
	ret = XAllocNamedColor( X11GD[GD_act]->disp, X11GD[GD_act]->colmap,
		colname,
		&actualColor,&exactColor );
	if (ret == 0)
	 { fprintf(stderr,"SGL_COLORDEF: Can't allocate color (%s).\n",
		colname);
	   return(-1);
	 }
	else
	 { color_pixel[color_stack_top] = actualColor.pixel;
	   XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
			color_pixel[color_stack_top] );
	   color_foreground = color_stack_top;
	   if (color_stack_top >= COLOR_MAX)
	    { fprintf(stderr,"SGL_COLORDEF: %d colors available only.\n",
		COLOR_MAX);
	      return(-1);
	    }
	   else
	    { color_stack_top++;
	      return_value = (color_stack_top-1);
	    }
	 }
	break;
    }
  return(return_value);
}
 


/************************************************************************/
/************************************************************************/
/*									*/
/* MISC. ...								*/
/*									*/
/************************************************************************/
/************************************************************************/


/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_debugging(int)					*/
/*		set flag, print debugging information or not		*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_debugging(int value)
#else
void	sgl_debugging(value)
	int	value;
#endif
{
 if (value == 0)	debugging = False;
 else			debugging = True;
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	FatalError(char *)					*/
/*		printf fatal error message and exit			*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	FatalError(char *str)
#else
void	FatalError(str)
	char	*str;
#endif
{
 fprintf(stderr,"%s",str);
 exit(-1);
}

