/************************************************************************/
/*									*/
/*	SGL.H					(C) 9/94 Ch. Muenkel	*/
/*	Include file for libsgl.a library				*/
/*									*/
/************************************************************************/
/*									*/
/*	Simple Graphics Library						*/
/*	An easy to use interface to the X11 window system		*/
/*									*/
/************************************************************************/
/*									*/
/*	Author:	Christian Muenkel					*/
/*		Present address:					*/
/*		muenkel@carbo.tphys.uni-heidelberg.de			*/
/*		Christian Muenkel					*/
/*		Institut fuer Theoretische Physik			*/
/*		Philosophenweg 19					*/
/*		69120 Heidelberg - Germany				*/
/*									*/
/************************************************************************/
/* version      date            comment                                 */
/*                                                                      */
/* V1.0		 1/94		first implementation 			*/
/* 		 9/94		final tests				*/
/*                                                                      */
/************************************************************************/

#ifndef		__SGL_INCLUDED
#define		__SGL_INCLUDED

#include        <stdio.h>
#include        <stdlib.h>
#include        <math.h>


#include        <X11/Xlib.h>
#include        <X11/Xutil.h>


/************************************************************************/
/* global defines, which should be accesible from all modules		*/
/************************************************************************/
#define	VISTYPE_StaticGray		1
#define	VISTYPE_StaticColor		2
#define	VISTYPE_TrueColor		3
#define	VISTYPE_GrayScale		4
#define	VISTYPE_PseudoColor		5
#define	VISTYPE_DirectColor		6

#define	SGL_BLACK	0
#define	SGL_WHITE	1

/************************************************************************/
/* function declaration							*/
/************************************************************************/
#ifdef		__STDC__				/* ANSI C	*/
void		sgl_init(void);
void		sgl_initm(int);

void		sgl_viewport(float,float,float,float);
void		sgl_winsize(int,int);
void		sgl_display(void);
void		sgl_debugging(int);

int		sgl_colordef(char *);
void		sgl_color(int);
void		sgl_color_background(int);

void		sgl_line(float,float,float,float);
void		sgl_circle(float,float,float);
void		sgl_disc(float,float,float);
void		sgl_rectangle(float,float,float,float);
void		sgl_rectarea(float,float,float,float);

void		sgl_select_win(int);
void		sgl_map_win(int);
void		sgl_unmap_win(int);
void		sgl_pos_win(int,int);

void		FatalError(char *);

#else

void		sgl_init();
void		sgl_initm();

void		sgl_viewport();
void		sgl_winsize();
void		sgl_display();
void		sgl_debugging();

int		sgl_colordef();
void		sgl_color();
void		sgl_color_background();

void		sgl_line();
void		sgl_circle();
void		sgl_disc();
void		sgl_rectangle();
void		sgl_rectarea();

void		sgl_select_win();
void		sgl_map_win();
void		sgl_unmap_win();
void		sgl_pos_win();

void		FatalError();

#endif			/* of ifdef  __STDC__		*/

#endif			/* of ifndef __SGL_INCLUDED	*/
