/************************************************************************/
/*									*/
/*	SGLGLOB.H				(C) 9/94 Ch. Muenkel	*/
/*	Global data of SGL library					*/
/*									*/
/************************************************************************/
/*									*/
/*	Simple Graphics Library						*/
/*	An easy to use interface to the X11 window system		*/
/*									*/
/************************************************************************/
/*									*/
/*	Author:	Christian Muenkel					*/
/*		Present address:					*/
/*		muenkel@carbo.tphys.uni-heidelberg.de			*/
/*		Christian Muenkel					*/
/*		Institut fuer Theoretische Physik			*/
/*		Philosophenweg 19					*/
/*		69120 Heidelberg - Germany				*/
/*									*/
/************************************************************************/
/* version      date            comment                                 */
/*                                                                      */
/* V1.0		 1/94		first implementation 			*/
/* 		 9/94		final tests				*/
/*                                                                      */
/************************************************************************/

#include	"sgl.h"

#define	MAX_X11GD_STRUCT	16
#define	COLOR_MAX		256

/************************************************************************/
/* X11 related global data, which should be accesible from all modules	*/
/************************************************************************/
struct X11GlobalData {
	Display		*disp;		/* connection to X server	*/
	char		*disp_name;	/* hardware display name	*/
        Screen		*scrn;		/* screen of display <disp>	*/
	int		scrnn;		/* number of screen		*/
	Window		win;		/* number of window		*/
	GC		gc;		/* graphic context		*/
	Visual		*vis;		/* visual type			*/
        Pixmap		pm;		/* background image struct	*/
	int		vistype;	/* type of visual (see defines)	*/
	int		depth;		/* bits per pixel of display	*/
	Colormap	colmap;		/* colormap of display		*/
	int		win_sx,win_sy;	/* window size in pixels	*/
	int		win_x_min,win_y_min;	/* left / bottom border	*/
	int		win_x_max,win_y_max;	/* right / top border	*/
	int		win_idx,win_idy;	/* inverse size		*/
	};

#ifdef	__SGL_MAIN
struct X11GlobalData *(X11GD[MAX_X11GD_STRUCT]);	
#else
extern struct X11GlobalData *(X11GD[MAX_X11GD_STRUCT]);	
#endif

/************************************************************************/
/* global data, which should be accesible from all modules		*/
/************************************************************************/
#ifdef	__SGL_MAIN
int	win_sx=256,win_sy=256;			/* window size in pixels*/
int	win_px=0,win_py=0;			/* preference position	*/
float	win_x_min,win_x_max;			/* viewport coord.	*/
float	win_y_min,win_y_max;			/*   vertical		*/
float	win_idx,win_idy;			/* inverse size		*/
int	GD_act;					/* active window	*/
int	GD_num;					/* number of windows	*/

int	flag_initialized = False;		/* not initialized	*/
int	debugging = False;			/* print debugging inf.	*/

int	color_foreground;			/* activ color		*/
int	color_background;			/* background color	*/
int	color_stack_top;			/* color stack pointer	*/
unsigned long	*color_pixel;			/* pixel values of color*/

#else

extern int	win_sx,win_sy;			/* window size in pixels*/
extern int	win_px,win_py;			/* preference position	*/
extern float	win_x_min,win_x_max;		/* viewport coord.	*/
extern float	win_y_min,win_y_max;		/*    vertical		*/
extern float	win_idx,win_idy;		/* inverse size		*/
extern int	GD_act;				/* active window	*/
extern int	GD_num;				/* number of windows	*/

extern int	flag_initialized;		/* not initialized	*/
extern int	debugging;			/* print debugging inf.	*/

extern int	color_foreground;		/* activ color		*/
extern int	color_background;		/* background color	*/
extern int	color_stack_top;
extern unsigned long	*color_pixel;		/* pixel values of color*/

#endif
